import re

import markdown
from django.conf import settings
from django.template.loader import render_to_string

from .base import MarkymarkExtension


class AutoLinkPostprocessor(markdown.postprocessors.Postprocessor):
    """
    Post processor to look for valid URIs and converts them to html links.
    """

    AUTOLINK_RE = re.compile(
        (
            r'(href="|src="|<a.*>)?'
            r'(?:(https?|ftps?|file|ssh|mms|svn(?:\+ssh)?|git|dict|nntp|irc|'
            r'rsync|smb|apt|telnet|s?news|sips?|skype|apt)://|(mailto:))'
            r'([-A-Za-z0-9+&@#/%?=~_()|!:,.;]*[-A-Za-z0-9+&@#/%=~_()|])'
        )
    )

    def run(self, text):
        def re_callback(match):
            prefix = match.group(1) or ''

            # Skip already rendered links/real html links.
            if prefix in ('href="', 'src="') or prefix.startswith('<a'):
                return match.group()

            if match.group(3) == 'mailto:':
                name = match.group(4)
            else:
                name = match.group()

            return render_to_string(
                getattr(settings, 'MARKYMARK_TEMPLATE_AUTOLINK', 'markymark/autolink.html'),
                {'url': match.group(), 'name': name},
            ).strip()

        return self.AUTOLINK_RE.sub(re_callback, text)


class AutoLinkExtension(MarkymarkExtension):
    """
    Extension to insert html links for certain URIs.
    """

    postprocessors = (AutoLinkPostprocessor,)


def makeExtension(**kwargs):
    return AutoLinkExtension(**kwargs)



import re

import markdown
from anylink.models import AnyLink
from django.conf import settings
from django.template.loader import render_to_string

from .base import MarkymarkExtension


class AnyLinkPostprocessor(markdown.postprocessors.Postprocessor):
    """
    Post processor to look for anylink link tags, replaces these tags with html links.
    """

    LINK_RE = re.compile(r'(\[link\:(?P<id>\d+)\])', re.IGNORECASE)

    def run(self, text):
        def re_callback(match):
            options = match.groupdict()
            try:
                link = AnyLink.objects.get(pk=int(options['id']))
                return render_to_string(
                    getattr(settings, 'MARKYMARK_TEMPLATE_ANYLINK', 'markymark/anylink.html'),
                    {'link': link},
                ).strip()

            except (KeyError, AnyLink.DoesNotExist):
                if settings.DEBUG:
                    raise

            return match.group(0).replace(match.group(1), '')

        return self.LINK_RE.sub(re_callback, text)


class AnyLinkExtension(MarkymarkExtension):
    """
    Extension to insert django-anylink links into a markdown document.
    """

    postprocessors = (AnyLinkPostprocessor,)

    class Media:
        js = ('markymark/extensions/anylink.js',)


def makeExtension(**kwargs):
    return AnyLinkExtension(**kwargs)



from django import forms
from django.db import models

from .widgets import MarkdownTextarea


class MarkdownFormField(forms.CharField):
    """
    Form field to configure the markdown textarea widget if not already set.
    """

    def __init__(self, *args, **kwargs):
        """
        Update the kwargs to set the markdown widget.
        Special note: the provided widget should be a subclass of MarkdownTextarea,
        if not the provided widget will be ignored.
        """
        widget = kwargs.get('widget', MarkdownTextarea)
        try:
            if not issubclass(widget, MarkdownTextarea):
                widget = MarkdownTextarea
        except TypeError:
            if not isinstance(widget, MarkdownTextarea):
                widget = MarkdownTextarea

        kwargs['widget'] = widget
        super().__init__(*args, **kwargs)


class MarkdownField(models.TextField):
    """
    Model field based on TextField with enabled markdown form field.
    """

    def formfield(self, form_class=MarkdownFormField, **kwargs):
        return super().formfield(form_class=form_class, **kwargs)



from markymark.extensions.base import MarkymarkExtension


class CleanExtension(MarkymarkExtension):
    """
    Extension to enable the cleanup plugin for the markdown editor.
    """

    class Media:
        js = ('markymark/extensions/clean.js',)


def makeExtension(**kwargs):
    return CleanExtension(**kwargs)



from django import template

from ..renderer import render_markdown


register = template.Library()


@register.filter(is_safe=True, name='markdown')
def markdown_filter(value, extensions=None):
    """
    Template which converts a provided value using markdown to html.
    Accepts additional extensions when rendering with markdown.
    """
    return render_markdown(value, extensions)



from django import forms
from django.conf import settings

from markymark.renderer import initialize_renderer


class MarkdownTextarea(forms.Textarea):
    """
    Extended forms Textarea which enables the javascript markdown editor.
    """

    def __init__(self, *args, **kwargs):
        """
        Sets the required data attributes to enable the markdown editor.
        """
        super().__init__(*args, **kwargs)
        self.attrs['data-provide'] = 'markdown'

        if getattr(settings, 'MARKYMARK_ICONLIBRARY', None):
            self.attrs['data-iconlibrary'] = settings.MARKYMARK_ICONLIBRARY

    @property
    def media(self):
        """
        Returns a forms.Media instance with the basic editor media and media
        from all registered extensions.
        """
        css = ['markymark/css/markdown-editor.css']
        iconlibrary_css = getattr(
            settings, 'MARKYMARK_FONTAWESOME_CSS', 'markymark/fontawesome/fontawesome.min.css'
        )
        if iconlibrary_css:
            css.append(iconlibrary_css)

        media = forms.Media(css={'all': css}, js=('markymark/js/markdown-editor.js',))

        # Use official extension loading to initialize all extensions
        # and hook in extension-defined media files.
        renderer = initialize_renderer()

        for extension in renderer.registeredExtensions:
            if hasattr(extension, 'media'):
                media += extension.media
        return media



import re

import markdown
from django.conf import settings
from django.template.loader import render_to_string
from filer.models import File

from .base import MarkymarkExtension


class FilerPostprocessor(markdown.postprocessors.Postprocessor):
    """
    Filer markdown extension for django-filer to show files and images.
    """

    FILE_RE = re.compile(r'(\[file\:(?P<id>\d+)\])', re.IGNORECASE)

    def run(self, text):
        def re_callback(match):
            options = match.groupdict()
            try:
                obj = File.objects.get(pk=int(options['id']))
                return render_to_string(
                    getattr(settings, 'MARKYMARK_TEMPLATE_FILER', 'markymark/filer.html'),
                    {'file': obj.get_real_instance()},
                ).strip()

            except (KeyError, File.DoesNotExist):
                if settings.DEBUG:
                    raise

            return match.group(0).replace(match.group(1), '')

        return self.FILE_RE.sub(re_callback, text)


class FilerExtension(MarkymarkExtension):
    """
    Extension to look for file tags, replaces them with html tags.
    In case of image, the image is added as img-tag, files are added as download links.
    """

    postprocessors = (FilerPostprocessor,)

    class Media:
        js = ('markymark/extensions/filer.js',)
        css = {'all': ('markymark/extensions/filer.css',)}


def makeExtension(**kwargs):
    return FilerExtension(**kwargs)



import markdown
from django.conf import settings
from django.utils.encoding import force_str
from django.utils.safestring import mark_safe


DEFAULT_MARKYMARK_EXTENSIONS = [
    'markymark.extensions.autolink',
]


def initialize_renderer(extensions=None):
    """
    Initializes the renderer by setting up the extensions (taking a comma separated
    string or iterable of extensions). These extensions are added alongside with the
    configured always-on extensions.

    Returns a markdown renderer instance.
    """
    if extensions is None:
        extensions = []

    if isinstance(extensions, str):
        extensions = [extension.strip() for extension in extensions.split(',')]

    for extension in getattr(settings, 'MARKYMARK_EXTENSIONS', DEFAULT_MARKYMARK_EXTENSIONS):
        extensions.append(extension)

    return markdown.Markdown(extensions=extensions)


def render_markdown(value, extensions=None):
    """
    Takes a text and a optional list of extensions and returns the rendered markdown text.

    The result is marked safe.
    """
    return mark_safe(initialize_renderer(extensions).convert(force_str(value)))



import markdown
from django.forms.widgets import MediaDefiningClass


class MarkymarkExtension(markdown.Extension, metaclass=MediaDefiningClass):
    """
    Base class for all markymark extensions.

    Actually its just a markdown.Extension class with some media attached.
    """

    preprocessors = None
    inlinepatterns = None
    postprocessors = None

    def extendMarkdown(self, md):
        """
        Every extension requires a extendMarkdown method to tell the markdown
        renderer how use the extension.
        """
        md.registerExtension(self)

        for processor in self.preprocessors or []:
            md.preprocessors.register(processor(md), processor.__name__.lower(), 0)

        for pattern in self.inlinepatterns or []:
            md.inlinePatterns.register(pattern(md), pattern.__name__.lower(), 0)

        for processor in self.postprocessors or []:
            md.postprocessors.register(processor(md), processor.__name__.lower(), 0)
